#!/bin/bash
#set -nv
#set -x
#
# Web-Based Enterprise Service Utilities
#   containing utilities functions for WEBES 4.5
#
# USAGE:  included and used by various WEBES installation and configuration scripts.
#
# Copyright 1999 Compaq Computer Corporation.  COMPAQ, the Compaq logo and the
# Digital logo are registered in the U.S. Patent and Trademark Office.  Pro-
# prietary service tool software. Valid license from Compaq or authorized
# sublicensor required for possession, use or copying. Consistent with FAR
# 12.211 and 12.212, Commercial Computer Software, Computer Software Documenta-
# tion, and Technical Data for Commercial Items are licensed to the U.S. Government
# under vendor's standard commercial license.
#
# Compaq service tool software, including associated documentation, is the property
# of and contains confidential technology of Compaq Computer Corporation.  Service
# customer is hereby licensed to use the software only for activities directly relating
# to the delivery of, and only during the term of, the applicable services delivered
# by Compaq or its authorized service provider.  Customer may not modify or reverse
# engineer, remove or transfer the software or make the software or any resultant
# diagnosis or system management data available to other parties without Compaq's or
# its authorized service provider's consent.  Upon termination of the services customer
# will, at Compaq's or its service provider's option, destroy or return the software and
# associated documentation in its possession./bin/egrep
#
# Compaq Web-Based Enterprise Service Suite V4.5 utilities for HP Linux
#
# This file is intended to be included in the other WEBES sscripts, using the
# syntax:
#              . $SVCTOOLS_HOME/webes/bin/wcc_install_443.sh
#
# If the file is not found, no error occurs until an attempt is made to use
# a function from the file. Then  you get a function "not found" error.
#
# Prerequisites: each function lists its own prereq's, but all expect $SVCTOOLS_HOME and
# $INSTDIR to be defined.
#
###############################################################################

# Search the cluster to see where the Director is running, i.e., search the
# /opt/hp/svctools/specific/desta/data/DESTA.stat file to see if the state is
# SYSTEM_STARTING or SYSTEM_UP. Exports $RUNNING_NODES and $RUNNING_MBRS.
# Note that $RUNNING_MBRS can be null when accessed from SCPs.
#
# NOTE: Expects IS_CLUSTER_MEMBER and MY_MEMBER_ID, so this may not be appropriate for SCPs.
#
# $1 = ALL_NODES or null. Check only local node if null.

# Make all new files have no permissions for group and world.
umask 077

check_WEBES_running ()
{
   WCC_RUNNING_NODES=""; WCC_RUNNING_MBRS=""

   check_for_local_WEBES

   if [ "$1" = "ALL_NODES" -a -n "$IS_CLUSTER_MEMBER" ]
   then
      # If we're clustered, see if it's running on any of the other nodes. The
      # /opt/hp/svctools/specific/desta/data/DESTA.stat file will contain these
      # states: SYSTEM_STARTING, SYSTEM_UP, SYSTEM_DOWN, or SYSTEM_SHUTDOWN.

      [ -z "$OTHER_MBRS_LIST" ] && get_other_cluster_members

      for CWR_MBR in $OTHER_MBRS_LIST; do
         check_for_member_WEBES $CWR_MBR
      done
   fi

   # Transfer the working lists to the user lists.
   RUNNING_NODES=""; RUNNING_MBRS=""
   for TEMP in $WCC_RUNNING_NODES; do
      if [ -z "$RUNNING_NODES" ]; then
         RUNNING_NODES=$TEMP
      else
         RUNNING_NODES="$TEMP $RUNNING_NODES"
      fi
   done
   for TEMP in $WCC_RUNNING_MBRS; do
      if [ -z "$RUNNING_MBRS" ]; then
         RUNNING_MBRS=$TEMP
      else
         RUNNING_MBRS="$TEMP $RUNNING_MBRS"
      fi
   done

   if [ -z "$RUNNING_MBRS" -a -z "$RUNNING_NODES" ]; then
      return 0 
   else
      return 1
   fi
}

# Add to WCC_RUNNING_NODES and WCC_RUNNING_MBRS the local node IDs if
# a local Desta process is found. Callers must initialize the lists.

check_for_local_WEBES ()
{
   # Check for new-style processes, and old-style if none found.

   TEMP="`/bin/ps -ef | /bin/egrep 'svctools.common.jre' | /bin/grep -v grep`"
   if [ -n "$TEMP" ]; then
      if [ -z "$WCC_RUNNING_NODES" ]; then
         WCC_RUNNING_NODES=$FULL_HOSTNAME
      else
         WCC_RUNNING_NODES="$FULL_HOSTNAME $WCC_RUNNING_NODES"
      fi
      if [ -z "$WCC_RUNNING_MBRS" ]; then
         WCC_RUNNING_MBRS=$MY_MEMBER_ID
      else
         WCC_RUNNING_MBRS="$MY_MEMBER_ID $WCC_RUNNING_MBRS"
      fi
   else
      TEMP="`/bin/ps -ef | /bin/grep desta_exec | /bin/grep -v grep | /bin/awk '{print $2}'`"
      if [ -n "$TEMP" ]; then
         if [ -z "$WCC_RUNNING_NODES" ]; then
            WCC_RUNNING_NODES=$FULL_HOSTNAME
         else
            WCC_RUNNING_NODES="$FULL_HOSTNAME $WCC_RUNNING_NODES"
	 fi
	 if [ -z "$WCC_RUNNING_MBRS" ]; then
            WCC_RUNNING_MBRS=$MY_MEMBER_ID
         else
            WCC_RUNNING_MBRS="$MY_MEMBER_ID $WCC_RUNNING_MBRS"
	 fi
      fi
   fi
}

# Add to WCC_RUNNING_NODES and WCC_RUNNING_MBRS the remote member's IDs if
# a Desta process is found. The /opt/hp/svctools/specific/desta/data/DESTA.stat 
# file will contain these states: SYSTEM_STARTING, SYSTEM_UP, SYSTEM_DOWN, 
# SYSTEM_SHUTDOWN or SYSTEM_ERROR.
# $1 = memberID of the node to check

check_for_member_WEBES ()
{
    STATFILE="`/bin/ls $INSTDIR/usr/cluster/members/$1/opt/hp/svctools/specific/desta/data/DESTA.stat 2> /dev/null`"
    if [ -n "${STATFILE}" ]; then
       UP="`/bin/grep "SYSTEM_" ${STATFILE} | /bin/grep -v DOWN | /bin/grep -v ERROR`"
       if [ -n "${UP}" ]; then
          if [ -z "$WCC_RUNNING_MBRS" ]; then
	     WCC_RUNNING_MBRS=$1
          else
	     WCC_RUNNING_MBRS="$1 $WCC_RUNNING_MBRS"
	  fi
	  TEMP=`/bin/grep "$1=" $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes | /bin/cut -f 2 -d=`
	  if [ -n "$TEMP" ]; then
	     if [ -z "$WCC_RUNNING_NODES" ]; then
	        WCC_RUNNING_NODES=$TEMP
	     else
	        WCC_RUNNING_NODES="$TEMP $WCC_RUNNING_NODES"
	     fi
	  else
	     /bin/echo "check_for_member_WEBES: unable to translate $1 to node name"
	  fi
       fi
    fi
}

#Create the temp file and link it to desta if optional components are not
#installed

create_temp_desta_file ()
{
    SBINFILES="desta"
    if [ -e ${SVCTOOLS_HOME}/common/bin/temp_desta_message ]; then
        rm ${SVCTOOLS_HOME}/common/bin/temp_desta_message
    fi
    /bin/touch ${SVCTOOLS_HOME}/common/bin/temp_desta_message
    /bin/cat <<EOF >>$SVCTOOLS_HOME/common/bin/temp_desta_message
    /bin/echo "The desta command is not enabled. Enable this command by"
    /bin/echo "installing and configuring one of the optional components"
    /bin/echo ""
    /bin/echo "System Event Analyzer ${PREFIX}${CADOTVER}"
    /bin/echo "Computer Crash Analysis Tool ${PREFIX}${CCATDOTVER}"
    /bin/echo ""
    /bin/echo "The WEBES configuration and installation tool's path"
    /bin/echo "and filename is: /usr/sbin/webes_install_update"
    /bin/echo ""  

EOF

    /bin/chmod +x ${SVCTOOLS_HOME}/common/bin/temp_desta_message
    for i in $SBINFILES; do
      /bin/ln -sf ${SVCTOOLS_HOME}/common/bin/temp_desta_message /usr/sbin/$i
    done
}

# Add to WCC_RUNNING_NODES and WCC_RUNNING_MBRS the remote member's IDs if
# a Desta process is found. The /opt/hp/svctools/specific/desta/data/DESTA.stat 
# file will contain these states: SYSTEM_STARTING, SYSTEM_UP, SYSTEM_DOWN, 
# SYSTEM_SHUTDOWN or SYSTEM_ERROR.
# $1 = node name of the node to check

check_for_node_WEBES ()
{
    TEMP=`/bin/egrep "$1$" $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes | /bin/cut -f 1 -d=`
    if [ -z "$TEMP" ]; then
       /bin/echo "check_for_node_WEBES: unable to translate $1 to cluster member ID"
       return
    fi
    STATFILE="`/bin/ls $INSTDIR/usr/cluster/members/$TEMP/opt/hp/svctools/specific/desta/data/DESTA.stat 2> /dev/null`"
    if [ -n "${STATFILE}" ]; then
       UP="`/bin/grep "SYSTEM_" ${STATFILE} | /bin/grep -v DOWN | /bin/grep -v ERROR`"
       if [ -n "${UP}" ]; then
          if [ -z "$WCC_RUNNING_NODES" ]; then
	     WCC_RUNNING_NODES=$1
	  else
             WCC_RUNNING_NODES="$1 $WCC_RUNNING_NODES"
	  fi
	  if [ -z "$WCC_RUNNING_MBRS" ]; then
	     WCC_RUNNING_MBRS=$TEMP
	  else
	     WCC_RUNNING_MBRS="$TEMP $WCC_RUNNING_MBRS"
	  fi
       fi
    fi
}

get_supported_ISEE ()
{
    ISEE_INSTALLED=""
    get_installed_product ISEE_INSTALLED ""
    if [ -n "$ISEE_INSTALLED" ]; then
      if [ "$ISEE_INSTALLED" -lt 350 ]; then
             ISEE_INSTALLED=""
        fi
    fi
}

###############################################################################
# These functions stop or start WEBES on local or remote nodes.

# The desta_process_stop and start functions work on the local node only.
# They issue a request then wait for 10 seconds for it to complete. The 
# stop function then kills all existing Desta processes, including CLI sessions.

desta_process_start ()
{
   SLEEPS=0; DPS_STATUS=0
   while true
   do
      WCC_RUNNING_NODES=""; WCC_RUNNING_MBRS=""
      check_for_local_WEBES
      [ -n "$WCC_RUNNING_NODES" ] && break
      if [ $SLEEPS = 0 ]; then
         $INSTDIR/usr/sbin/desta start
	 SLEEPS=`expr $SLEEPS + 1`
      else
         if [ $SLEEPS -gt 5 ]; then
	    /bin/echo "....could not start the Director on $FULL_HOSTNAME"
	    DPS_STATUS=1
	    break
	 fi
         sleep 2
         SLEEPS=`expr $SLEEPS + 1`
      fi
   done
   eval $1=$DPS_STATUS
}

desta_process_stop ()
{
   SLEEPS=0
   while true
   do
      WCC_RUNNING_NODES=""; WCC_RUNNING_MBRS=""
      check_for_local_WEBES
      [ -z "$WCC_RUNNING_NODES" ] && break
      if [ $SLEEPS = 0 ]; then
         $INSTDIR/usr/sbin/desta stop
	 SLEEPS=`expr $SLEEPS + 1`
      else
         if [ $SLEEPS -gt 5 ]; then
	    # get a list of processes and kill them all
	    /bin/echo "desta_process_stop: stop request failed, killing all processes"
	    TEMP="`/bin/ps -ef | /bin/egrep 'svctools.common.jre' | /bin/grep -v grep | /bin/awk '{print $2}'`"
	    if [ -n "$TEMP" ]; then
	       # there are new-style processes
	       for PROCESS in $TEMP; do
		  kill -9 ${PROCESS}
	       done
	    else
	       # check for old-style processes
	       TEMP="`/bin/ps -ef | /bin/grep desta_exec | /bin/grep -v grep | /bin/awk '{print $2}'`"
	       if [ -n "$TEMP" ]; then
		  for PROCESS in $TEMP; do
		     kill -9 ${PROCESS}
		  done
	       fi
	    fi
	    break
	 fi
         sleep 2
         SLEEPS=`expr $SLEEPS + 1`
      fi
   done
   eval $1=0
}

# These functions start and stop the Director on a remote node. 
# They expect $MBR and $NODE to be defined.

desta_remote_start ()
{
   SLEEPS=0; DPS_STATUS=0
   while true
   do
      WCC_RUNNING_NODES=""; WCC_RUNNING_MBRS=""
      check_for_member_WEBES $MBR
      [ -n "$WCC_RUNNING_MBRS" ] && break
      if [ $SLEEPS = 0 ]; then
         execute_on_NODE /usr/sbin/desta start
	 SLEEPS=`expr $SLEEPS + 1`
      else
         if [ $SLEEPS -gt 5 ]; then
	    /bin/echo "....could not start WEBES on $NODE"
	    DPS_STATUS=1
	    break
	 fi
         sleep 2
         SLEEPS=`expr $SLEEPS + 1`
      fi
   done
   eval $1=$DPS_STATUS
}

desta_remote_stop ()
{
   SLEEPS=0; DPS_STATUS=0
   while true
   do
      WCC_RUNNING_NODES=""; WCC_RUNNING_MBRS=""
      check_for_member_WEBES $MBR
      [ -z "$WCC_RUNNING_MBRS" ] && break
      if [ $SLEEPS = 0 ]; then
         execute_on_NODE /usr/sbin/desta stop
	 SLEEPS=`expr $SLEEPS + 1`
      else
         if [ $SLEEPS -gt 5 ]; then
	    /bin/echo "....could not stop WEBES on $NODE"
	    DPS_STATUS=1
	    break
	 fi
         sleep 2
         SLEEPS=`expr $SLEEPS + 1`
      fi
   done
   eval $1=$DPS_STATUS
}

###############################################################################
# Functions to determine what components are installed.

get_all_installed_products ()
{
    get_installed_product WEBES_INSTALLED ""; export WEBES_INSTALLED
    get_installed_product CA_INSTALLED ""; export CA_INSTALLED
    get_installed_product CCAT_INSTALLED ""; export CCAT_INSTALLED
#    get_installed_product UNICEN_INSTALLED ""; export UNICEN_INSTALLED
    get_installed_product DSN_INSTALLED ""; export DSN_INSTALLED
    get_installed_product ISEE_INSTALLED ""; export ISEE_INSTALLED
}

# Returns non-dotted version number, e.g., 123, if $2 not supplied. If $2 exists
# a dotted version is returned. The Unicensus dotted format is 1.2.3 while all
# other products get 1.23

get_installed_product ()
{

   INSTALLED_PRODUCT=""

   if [ "$1" = "WEBES_INSTALLED" ]; then
      BASE_PRODUCT="WEBESBASE"
   elif [ "$1" = "CCAT_INSTALLED" ]; then
      BASE_PRODUCT="WEBESCCAT"
   elif [ "$1" = "DSN_INSTALLED" ]; then
      BASE_PRODUCT="DSNABASE"
   elif [ "$1" = "ISEE_INSTALLED" ]; then
      BASE_PRODUCT="ISEEPlatform"
   else
      BASE_PRODUCT="WEBESSEA"
   fi

    PKG=`rpm -aq|grep ^${BASE_PRODUCT}`
 if [ "$PKG" != "" ]; then
    INSTALLED_PRODUCT=`echo $PKG 2> /dev/null | /bin/cut -f2 -d "-"`
 elif [ "$PKG" = "ISEEPlatform" ]; then
    INSTALLED_PRODUCT=`echo $PKG 2> /dev/null | /bin/cut -f3 -d "." | /bin/cut -c1,2,3`
 fi

    eval $1=${INSTALLED_PRODUCT}
}

###############################################################################
###THIS FUNCTION IS COMMENTED DUE TO NON-FEASIBILITY OF PROTING TO LINUX#######
###############################################################################
# Functions in this section learn things about the system.

get_system_serial_number ()
{
  # If the system serial number is defined, then this will find it. Returns  
  # null if the serial number is not found.

  # SERNUM=`getconf  CS_MACHINE_SERIAL 2> /dev/null`

  # if [ -z "${SERNUM}" ]; then
  #    echo "sel path system;info;wait;infolog view done" | cstm 2> /dev/null > info
  #    SERNUM=`cat info | grep "System Serial" 2> /dev/null`
  #    SERNUM=`echo $SERNUM |cut -d : -f 2 2> /dev/null`
  #    rm -f info
  # fi

  # if [ -z "${SERNUM}" ]; then
  #     if [ -f /var/opt/ignite/local/manifest/manifest.seed ]; then
  #        SERNUM=`cat /var/opt/ignite/local/manifest/manifest.seed | grep "SERIAL" 2> /dev/null`
  #        SERNUM=`echo $SERNUM |cut -d " " -f 2 2> /dev/null`
  #     fi
  # fi

   eval "$1"="${SERNUM}"

}

###############################################################################
# Get the system name in all its glorious forms. Remember that some nodes don't
# have dot-notation, fully-qualified hostnames due to lazy system admin types.
# DO NOT get the host name from the registry... there's no telling where it came
# from.

get_hostnames ()
{
    FULL_HOSTNAME="`hostname`"
    #FULL_HOSTNAME="`/bin/grep $FULL_HOSTNAME /etc/hosts | /bin/awk '{print $2}'`"
    if [ -z "${FULL_HOSTNAME}" ]; then
	FULL_HOSTNAME="`/bin/uname -n`"
      # FULL_HOSTNAME="`/bin/grep $FULL_HOSTNAME /etc/hosts | /bin/awk '{print $2}'`"
    fi
    if [ -z "${FULL_HOSTNAME}" ]; then
	/bin/echo ""
	/bin/echo "....unable to determine local system's hostname"
	/bin/echo ""
	UPPER_HOSTNAME=""
	HOSTNAME=""
    else
	HOSTNAME="`/bin/echo $FULL_HOSTNAME | /bin/cut -f 1 -d.`"
	if [ -z "${HOSTNAME}" ]; then
	    HOSTNAME=$FULL_HOSTNAME
	fi
	UPPER_HOSTNAME="`/bin/echo ${HOSTNAME} | /usr/bin/tr '[:lower:]' '[:upper:]'`"
    fi
}

###############################################################################
# The functions in this section deal with cluster management, i.e., moving
# files, updating cluster configuration from one node, etc.


# Used to determine if the current node is part of a cluster. Sets the
# variable, IS_CLUSTER_MEMBER to null if it is not a member, and to "true"
# if it is.

isClusterMember ()
{
   IS_CLUSTER_MEMBER=""
   if [ -f ${INSTDIR}/usr/sbin/clu_get_info ]
   then
      ${INSTDIR}/usr/sbin/clu_get_info -q
      if [ "$?" = "0" ]; then
         IS_CLUSTER_MEMBER="true"
      fi
   fi
}

# Exports $OTHER_MBRS_LIST, a list of member IDs, e.g., member20 member21 member22...
# EXCEPT member0 and the local node. Expects MY_MEMBER_ID to be defined.

get_other_cluster_members ()
{
    OTHER_MBRS_LIST=""
    TEMP=`/bin/ls -1 ${INSTDIR}/usr/cluster/members | /bin/grep -e '[0-9]$'`
    for M in $TEMP; do
       [ "$M" = "member0" -o "$M" = "$MY_MEMBER_ID" ] && continue
       if [ -z "$OTHER_MBRS_LIST" ]; then
          OTHER_MBRS_LIST=$M
       else
          OTHER_MBRS_LIST="$M $OTHER_MBRS_LIST"
       fi
    done
    export OTHER_MBRS_LIST
}

# Exports $MBR_NODE_NAMES, a list of fully qualified node names for all nodes
# EXCEPT the member0 node. This includes $HOSTNAME. Uses $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes.

get_cluster_hostnames ()
{
    MBR_NODENAMES=""
    for LINE in `/bin/grep "=" $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes`
    do
	GCH_MBR=`/bin/echo $LINE | /bin/cut -f 1 -d=`
	[ "$GCH_MBR" = "member0" ] && continue
	NODE=`/bin/echo $LINE | /bin/cut -f 2 -d=`
	if [ -z "$MBR_NODENAMES" ]; then
	   MBR_NODENAMES=$NODE
	else
	   MBR_NODENAMES="$NODE $MBR_NODENAMES"
        fi
    done
    export MBR_NODENAMES
}

# Executes a program on all known nodes. Executes directly on the local node,
# and uses rsh to execute a program on the cluster nodes in $NODE_LIST. 
# Expects $IS_CLUSTER_MEMBER, $OTHER_MBRS_LIST, $NODE_LIST and $HOSTNAME to 
# be defined.
#
# $PROGRAM_NAME = path and name of program to execute, e.g., "./do_something"
# $* = parameters to be passed to $PROGRAM_NAME

execute_on_NODE_LIST ()
{
   # Don't call rsh at run level 2 (the network isn't up yet).
   # Separate the program name from the calling arguments.

   if [ "`/bin/who -r | /bin/awk '{print $3}'`" = "2" ];then
      /bin/echo ""
      /bin/echo "execute_on_NODE_LIST: cannot execute rsh at run level 2"
      /bin/echo 
      return 1
   fi

   ARGS=""
   PROGRAM_NAME=""
   for ARG in $*; do
     if [ -z "$PROGRAM_NAME" ]; then
        PROGRAM_NAME="$ARG"
     else
        ARGS="$ARGS $ARG"
     fi
   done

   # Loop to determine which of the requested nodes are in the UP state.
   # Execute the program on those nodes.

   for EN_NODE in $NODE_LIST; do
      /bin/echo "....executing on $EN_NODE"
      if [ "$EN_NODE" = "$FULL_HOSTNAME" ]; then
         # don't rsh to the local node
         $PROGRAM_NAME $ARGS
      else
         # don't rsh if the cluster member is down.
	 TEMP=`/bin/egrep "$EN_NODE$" $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes | /bin/cut -f 1 -d= | /bin/sed 's/member//'`
	 if [ -z "$TEMP" ]; then
	    /bin/echo "execute_on_NODE_LIST: unable to translate $EN_NODE to cluster member ID"
	    continue
	 fi
	 EN_UP=`/usr/sbin/clu_get_info -m $TEMP | /bin/grep state | /bin/grep UP`
	 if [ -z "$EN_UP" ]; then
	    /bin/echo ""
	    /bin/echo "....$EN_NODE is not currently up."
	    /bin/echo ""
	    continue
	 fi

	 # clu_get_info will show node is UP even when its
	 # network software is not yet up so ping it, too.
         [ -z "$PING_CMD" ] && set_ping_command
	 $PING_CMD $EN_NODE >/dev/null
	 if [ "$?" != "0" ]; then
	    /bin/echo ""
	    /bin/echo "....$EN_NODE does not respond to ping."
	    /bin/echo ""
	    continue
	 fi

	 # okay, do it
	 /bin/rsh -l root $EN_NODE $PROGRAM_NAME $ARGS
      fi
      STATUS=$?
      [ "$STATUS" != "0" ] && return $STATUS
   done
   /bin/echo ""
   return 0
}

# Executes a program on a specified node. Executes directly on the local node,
# and uses rsh to execute a program on a remote cluster node.
# Expects $IS_CLUSTER_MEMBER, $NODE and $HOSTNAME to be defined.
#
# $PROGRAM_NAME = path and name of program to execute, e.g., "./do_something"
# $* = parameters to be passed to $PROGRAM_NAME

execute_on_NODE ()
{
   # Don't call rsh at run level 2 (the network isn't up yet).
   # Separate the program name from the calling arguments.

   if [ "`/bin/who -r | /bin/awk '{print $3}'`" = "2" ];then
      /bin/echo ""
      /bin/echo "execute_on_NODE: cannot execute rsh at run level 2"
      /bin/echo 
      return 1
   fi

   ARGS=""
   PROGRAM_NAME=""
   for ARG in $*; do
     if [ -z "$PROGRAM_NAME" ]; then
        PROGRAM_NAME="$ARG"
     else
        ARGS="$ARGS $ARG"
     fi
   done

   # Execute the program - don't rsh to the local node.

   /bin/echo ""
   /bin/echo "....executing on $NODE"
   if [ "$NODE" = "$FULL_HOSTNAME" ]; then
      $PROGRAM_NAME $ARGS
      STATUS=$?
   else
      # don't rsh if the cluster member is down.
      TEMP=`/bin/egrep "$NODE$" $SVCTOOLS_HOME/common/desta/config/CommonFiles/member_nodes | /bin/cut -f 1 -d= | /bin/sed 's/member//'`
      if [ -z "$TEMP" ]; then
	 /bin/echo "execute_on_NODE: unable to translate $NODE to cluster member ID"
	 continue
      fi
      EN_UP=`/usr/sbin/clu_get_info -m $TEMP | /bin/grep state | /bin/grep UP`
      if [ -z "$EN_UP" ]; then
	/bin/echo ""
	/bin/echo "....$NODE is not currently up."
	/bin/echo ""
	continue
      fi

      # clu_get_info will show node is UP even when its
      # network software is not yet up so ping it, too.
      [ -z "$PING_CMD" ] && set_ping_command

      $PING_CMD $NODE >/dev/null
      STATUS=$?
      if [ "$STATUS" != "0" ]; then
	 /bin/echo ""
	 /bin/echo "....$NODE does not respond to ping."
	 /bin/echo ""	 
      else
	 /bin/rsh -l root $NODE $PROGRAM_NAME $ARGS
         STATUS=$?
      fi
   fi
   /bin/echo ""
   return $STATUS
}

PING_CMD=""

set_ping_command ()
{
   # Set the ping command - the -t option sets a timeout, but doesn't exist
   # until Unix v5.0.

   PING_CMD="/bin/ping -c1"

   get_registry_item common.OSmajor; OSMAJOR="$GR_ITEM"
   if [ "$OSMAJOR" -ge 5 ]; then
      get_registry_item common.OSminor; OSMINOR="$GR_ITEM"
      if [ "$OSMINOR" -ge 1  ]; then
         PING_CMD="/bin/ping -c1 -t3"
      else
         get_registry_item common.OSvariant; OSVARIANT="$GR_ITEM"
         get_registry_item common.OSbuild; OSBUILD="$GR_ITEM"
         if [ -n "$OSVARIANT" -o "$OSBUILD" -ge 910 ]; then
            PING_CMD="/bin/ping -c1 -t3"
         fi
      fi
   fi
}

###############################################################################
# The functions in this section deal with manipulating the registry file.
# Prerequisite: expects $REGISTRY_HOME to be defined.

# Create a tag=value pair in the registry.
# $1 = the tag, e.g., common.serialNumber.
# $2 = the value, e.g., "1234567890", or "" to remove the entry.

sed_registry_file ()
{
   ARGS=$#
   /bin/sed "s/$1.*$//g" ${REGISTRY_HOME}/DESTA.REG | /bin/grep -v '^ *$' > ${REGISTRY_HOME}/newfile
   mv ${REGISTRY_HOME}/newfile ${REGISTRY_HOME}/DESTA.REG
   if [ $ARGS -eq 2 ]; then
     /bin/echo "$1=$2" >> ${REGISTRY_HOME}/DESTA.REG
   fi
}

# Create tag-value pairs in a registry on another cluster node. If you have
# multiple items to enter it's more efficient to use sed_temp_registry and
# avoid the copies for each item.
# $1 = the memberID, e.g., member20
# $2 = the tag, e.g., common.serialNumber.
# $3 = the value, e.g., "1234567890", or "" to remove the entry.

sed_remote_registry_file ()
{
    /bin/cp -p $INSTDIR/usr/cluster/members/$1/opt/hp/svctools/specific/desta/config/DESTA.REG ./srr_temp
    if [ "$?" != "0" ]; then
	/bin/echo "sed_remote_registry: failed to retrieve registry from cluster member $1"
	return
    fi
    sed_temp_registry ./srr_temp $2 $3
    /bin/cp -p ./srr_temp $INSTDIR/usr/cluster/members/$1/opt/hp/svctools/specific/desta/config/DESTA.REG
    if [ "$?" != "0" ]; then
	/bin/echo "sed_remote_registry: failed to restore registry to cluster member $1"
    fi
    /bin/rm -f ./srr_temp
}

# Create tag-value pairs in a temporary registry file
# $1 = path and filename
# $2 = the tag, e.g., common.serialNumber.
# $3 = the value, e.g., "1234567890", or not supplied to remove the entry.

sed_temp_registry ()
{
   if [ -f $1 ]; then
      ARGS=$#
      /bin/sed "s/$2.*$//g" $1 | /bin/grep -v '^ *$' > ./tempfile
      mv ./tempfile $1
      if [ $ARGS -eq 3 ]; then
        /bin/echo "$2=$3" >> $1
      fi
   fi
}

# Update the registries on all cluster nodes. Expects $OTHER_MBRS_LIST
# and $IS_CLUSTER_MEMBER to be defined.
# $1 = the tag, e.g., common.serialNumber.
# $2 = the value, e.g., "1234567890", or "" to remove the entry.

sed_cluster_registries ()
{
   sed_registry_file $1 $2
   if [ -n "$IS_CLUSTER_MEMBER" ]; then
      for SCR_MBR in $OTHER_MBRS_LIST; do
         sed_remote_registry_file $SCR_MBR $1 $2
      done
   fi
}

# Retrieve the value of the specified tag.
# $1 = the tag name, e.g., common.serialNumber.

get_registry_item ()
{
   GR_ITEM=""
   GR_ITEM=`/bin/grep "$1" $REGISTRY_HOME/DESTA.REG | /bin/cut -f 2 -d=`

   # If this is a multi-word item the Java Properties class appends "\"
   # to all words but the last one. Strip these off.

   if [ -n "$GR_ITEM" ]; then
      if [ "`/bin/echo $GR_ITEM | /usr/bin/wc -w`" != "1" ]; then
         GR_ITEM=`/bin/echo $GR_ITEM | /bin/sed -e 's/\\\//g'`
      fi
   fi
}

# Retrieve a value from another cluster node.
# $1 = the tag name.
# $2 = the cluster member's ID, e.g., member20.

get_remote_registry_item ()
{
   GR_ITEM=""
   RRI=$INSTDIR/usr/cluster/members/$2/opt/hp/svctools/specific/desta/config
   if [ -f $RRI/DESTA.REG ]; then
      GR_ITEM=`/bin/grep "$1" $RRI/DESTA.REG | /bin/cut -f 2 -d=`
   fi
   if [ -n "$GR_ITEM" ]; then
      if [ "`/bin/echo $GR_ITEM | /usr/bin/wc -w`" != "1" ]; then
         GR_ITEM=`/bin/echo $GR_ITEM | /bin/sed -e 's/\\\//g'`
      fi
   fi
}

# Retrieve a value from a temporary registry file.
# $1 = path and filename
# $2 = the tag name, e.g., common.serialNumber.

get_temp_registry_item ()
{
   GR_ITEM=""
   GR_ITEM=`/bin/grep "$2" $1 | /bin/cut -f 2 -d=`
}

#########################################
get_answer_item()
{
   GA_ITEM=""
   GA_LINES=""; GA_TOKEN=""; GA_VALUE=""; GA_NODE=""

   [ "$#" = 1 -o "$#" = 2 -a -z "$2" ] && GA_NODE=$FULL_HOSTNAME
   [ "$#" = 2 -a -n "$2" ] && GA_NODE=$2
 
      GA_LINES=`/bin/grep $1 $ANSWER_FILE | /bin/grep -v "$1@" | /bin/grep -v "#"`
      if [ -z "$GA_LINES" ]; then
	 /bin/echo "....get_answer_item: ITEM MISSING - token ($1), node ($2)"
         GA_ITEM=""
	 return 1
      fi

   /bin/echo "....get_answer_item: token ($1), node ($2), recovered entry - $GA_LINES"

   GA_VALUE=`/bin/echo $GA_LINES | cut -f2 -d=`

   if [ -n "`/bin/echo $1 | /bin/grep Flag`" ]
   then
      # Dealing with a boolean - value is true | false | node list
      if [ -n "`/bin/echo $GA_VALUE | /bin/egrep '(true|false)'`" ]; then
	 # return true or false
	 GA_ITEM=$GA_VALUE
      else
         # node list - if node is in the list return true.
         if [ -n "`/bin/echo $GA_VALUE | /bin/grep $GA_NODE`" ]; then
	    GA_ITEM="true"
	 else
	    GA_ITEM="false"
	 fi
      fi
   else
      # Dealing with string data, return the value.
      GA_ITEM=$GA_VALUE
   fi
      /bin/echo "....get_answer_item: returning value - $GA_ITEM"
   return 0
}
#
##############################################
